@@ -1,49 +1,28 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2018-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
-
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Utilities and data structures used by various config code."""
 
 
-import typing
 import collections
 import itertools
 import operator
+from typing import (
+    TYPE_CHECKING, Any, Dict, Iterator, List, Optional, Sequence, Set, Union,
+    MutableMapping)
 
-from PyQt5.QtCore import QUrl
+from qutebrowser.qt.core import QUrl
+from qutebrowser.qt.gui import QFontDatabase
+from qutebrowser.qt.widgets import QApplication
 
-from qutebrowser.utils import utils, urlmatch, usertypes, qtutils
+from qutebrowser.utils import utils, urlmatch, urlutils, usertypes, qtutils
 from qutebrowser.config import configexc
 
-if typing.TYPE_CHECKING:
+if TYPE_CHECKING:
     from qutebrowser.config import configdata
 
 
-def _widened_hostnames(hostname: str) -> typing.Iterable[str]:
-    """A generator for widening string hostnames.
-
-    Ex: a.c.foo -> [a.c.foo, c.foo, foo]"""
-    while hostname:
-        yield hostname
-        hostname = hostname.partition(".")[-1]
-
-
 class ScopedValue:
 
     """A configuration value which is valid for a UrlPattern.
@@ -56,8 +35,8 @@ class ScopedValue:
 
     id_gen = itertools.count(0)
 
-    def __init__(self, value: typing.Any,
-                 pattern: typing.Optional[urlmatch.UrlPattern],
+    def __init__(self, value: Any,
+                 pattern: Optional[urlmatch.UrlPattern],
                  hide_userconfig: bool = False) -> None:
         self.value = value
         self.pattern = pattern
@@ -90,17 +69,17 @@ class Values:
         _domain_map: A mapping from hostnames to all associated ScopedValues.
     """
 
-    _VmapKeyType = typing.Optional[urlmatch.UrlPattern]
+    _VmapKeyType = Optional[urlmatch.UrlPattern]
 
     def __init__(self,
                  opt: 'configdata.Option',
-                 values: typing.Sequence[ScopedValue] = ()) -> None:
+                 values: Sequence[ScopedValue] = ()) -> None:
         self.opt = opt
-        self._vmap = collections.OrderedDict()  \
-            # type: collections.OrderedDict[Values._VmapKeyType, ScopedValue]
+        self._vmap: MutableMapping[
+            Values._VmapKeyType, ScopedValue] = collections.OrderedDict()
         # A map from domain parts to rules that fall under them.
-        self._domain_map = collections.defaultdict(set)  \
-            # type: typing.Dict[typing.Optional[str], typing.Set[ScopedValue]]
+        self._domain_map: Dict[
+            Optional[str], Set[ScopedValue]] = collections.defaultdict(set)
 
         for scoped in values:
             self._add_scoped(scoped)
@@ -117,7 +96,7 @@ class Values:
             return '\n'.join(lines)
         return '{}: <unchanged>'.format(self.opt.name)
 
-    def dump(self, include_hidden: bool = False) -> typing.Sequence[str]:
+    def dump(self, include_hidden: bool = False) -> Sequence[str]:
         """Dump all customizations for this value.
 
         Arguments:
@@ -138,7 +117,7 @@ class Values:
 
         return lines
 
-    def __iter__(self) -> typing.Iterator['ScopedValue']:
+    def __iter__(self) -> Iterator['ScopedValue']:
         """Yield ScopedValue elements.
 
         This yields in "normal" order, i.e. global and then first-set settings
@@ -151,12 +130,12 @@ class Values:
         return bool(self._vmap)
 
     def _check_pattern_support(
-            self, arg: typing.Union[urlmatch.UrlPattern, QUrl, None]) -> None:
+            self, arg: Union[urlmatch.UrlPattern, QUrl, None]) -> None:
         """Make sure patterns are supported if one was given."""
         if arg is not None and not self.opt.supports_pattern:
             raise configexc.NoPatternError(self.opt.name)
 
-    def add(self, value: typing.Any,
+    def add(self, value: Any,
             pattern: urlmatch.UrlPattern = None, *,
             hide_userconfig: bool = False) -> None:
         """Add a value with the given pattern to the list of values.
@@ -201,7 +180,7 @@ class Values:
         self._vmap.clear()
         self._domain_map.clear()
 
-    def _get_fallback(self, fallback: bool) -> typing.Any:
+    def _get_fallback(self, fallback: bool) -> Any:
         """Get the fallback global/default value."""
         if None in self._vmap:
             return self._vmap[None].value
@@ -211,8 +190,7 @@ class Values:
         else:
             return usertypes.UNSET
 
-    def get_for_url(self, url: QUrl = None, *,
-                    fallback: bool = True) -> typing.Any:
+    def get_for_url(self, url: QUrl = None, *, fallback: bool = True) -> Any:
         """Get a config value, falling back when needed.
 
         This first tries to find a value matching the URL (if given).
@@ -225,10 +203,10 @@ class Values:
             return self._get_fallback(fallback)
         qtutils.ensure_valid(url)
 
-        candidates = []  # type: typing.List[ScopedValue]
+        candidates: List[ScopedValue] = []
         # Urls trailing with '.' are equivalent to non-trailing types.
         # urlutils strips them, so in order to match we will need to as well.
-        widened_hosts = _widened_hostnames(url.host().rstrip('.'))
+        widened_hosts = urlutils.widened_hostnames(url.host().rstrip('.'))
         # We must check the 'None' key as well, in case any patterns that
         # did not have a domain match.
         for host in itertools.chain(widened_hosts, [None]):
@@ -247,8 +225,8 @@ class Values:
         return self._get_fallback(fallback)
 
     def get_for_pattern(self,
-                        pattern: typing.Optional[urlmatch.UrlPattern], *,
-                        fallback: bool = True) -> typing.Any:
+                        pattern: Optional[urlmatch.UrlPattern], *,
+                        fallback: bool = True) -> Any:
         """Get a value only if it's been overridden for the given pattern.
 
         This is useful when showing values to the user.
@@ -272,20 +250,23 @@ class FontFamilies:
 
     """A list of font family names."""
 
-    def __init__(self, families: typing.Sequence[str]) -> None:
+    def __init__(self, families: Sequence[str]) -> None:
         self._families = families
         self.family = families[0] if families else None
 
-    def __iter__(self) -> typing.Iterator[str]:
+    def __iter__(self) -> Iterator[str]:
         yield from self._families
 
+    def __len__(self) -> int:
+        return len(self._families)
+
     def __repr__(self) -> str:
         return utils.get_repr(self, families=self._families, constructor=True)
 
     def __str__(self) -> str:
         return self.to_str()
 
-    def _quoted_families(self) -> typing.Iterator[str]:
+    def _quoted_families(self) -> Iterator[str]:
         for f in self._families:
             needs_quoting = any(c in f for c in '., ')
             yield '"{}"'.format(f) if needs_quoting else f
@@ -294,6 +275,57 @@ class FontFamilies:
         families = self._quoted_families() if quote else self._families
         return ', '.join(families)
 
+    @classmethod
+    def from_system_default(
+            cls,
+            font_type: QFontDatabase.SystemFont = QFontDatabase.SystemFont.FixedFont,
+    ) -> 'FontFamilies':
+        """Get a FontFamilies object for the default system font.
+
+        By default, the monospace font is returned, though via the "font_type" argument,
+        other types can be requested as well.
+
+        Note that (at least) three ways of getting the default monospace font
+        exist:
+
+        1) f = QFont()
+           f.setStyleHint(QFont.StyleHint.Monospace)
+           print(f.defaultFamily())
+
+        2) f = QFont()
+           f.setStyleHint(QFont.StyleHint.TypeWriter)
+           print(f.defaultFamily())
+
+        3) f = QFontDatabase.systemFont(QFontDatabase.SystemFont.FixedFont)
+           print(f.family())
+
+        They yield different results depending on the OS:
+
+                QFont.StyleHint.Monospace  | QFont.StyleHint.TypeWriter | QFontDatabase
+                -----------------------------------------------------------------------
+        Win:    Courier New                | Courier New                | Courier New
+        Linux:  DejaVu Sans Mono           | DejaVu Sans Mono           | monospace
+        macOS:  Menlo                      | American Typewriter        | Monaco
+
+        Test script: https://p.cmpl.cc/076835c4
+
+        On Linux, it seems like both actually resolve to the same font.
+
+        On macOS, "American Typewriter" looks like it indeed tries to imitate a
+        typewriter, so it's not really a suitable UI font.
+
+        Looking at those Wikipedia articles:
+
+        https://en.wikipedia.org/wiki/Monaco_(typeface)
+        https://en.wikipedia.org/wiki/Menlo_(typeface)
+
+        the "right" choice isn't really obvious. Thus, let's go for the
+        QFontDatabase approach here, since it's by far the simplest one.
+        """
+        assert QApplication.instance() is not None
+        font = QFontDatabase.systemFont(font_type)
+        return cls([font.family()])
+
     @classmethod
     def from_str(cls, family_str: str) -> 'FontFamilies':
         """Parse a CSS-like string of font families."""
